package org.jboss.security.plugins;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.lang.reflect.Method;
import java.lang.reflect.UndeclaredThrowableException;
import java.security.Principal;
import java.security.acl.Group;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import org.jboss.logging.Logger;
import org.jboss.security.RealmMapping;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SubjectSecurityManager;
import org.jboss.security.auth.callback.SecurityAssociationHandler;
import org.jboss.security.plugins.SubjectActions;
import org.jboss.system.ServiceMBeanSupport;
import org.jboss.util.CachePolicy;
import org.jboss.util.TimedCachePolicy;

public class JaasSecurityManager extends ServiceMBeanSupport implements SubjectSecurityManager, RealmMapping {
  private String securityDomain;
  
  private CachePolicy domainCache;
  
  private CallbackHandler handler;
  
  private Method setSecurityInfo;
  
  protected Logger log;
  
  protected boolean trace;
  
  static Class class$org$jboss$security$plugins$JaasSecurityManager$DomainInfo;
  
  public JaasSecurityManager() {
    this("other", (CallbackHandler)new SecurityAssociationHandler());
  }
  
  public JaasSecurityManager(String securityDomain, CallbackHandler handler) {
    this.securityDomain = securityDomain;
    this.handler = handler;
    String categoryName = getClass().getName() + '.' + securityDomain;
    this.log = Logger.getLogger(categoryName);
    this.trace = this.log.isTraceEnabled();
    Class[] sig = { Principal.class, Object.class };
    try {
      this.setSecurityInfo = handler.getClass().getMethod("setSecurityInfo", sig);
    } catch (Exception e) {
      String msg = "Failed to find setSecurityInfo(Princpal, Object) method in handler";
      throw new UndeclaredThrowableException(e, msg);
    } 
    this.log.debug("CallbackHandler: " + handler);
  }
  
  public void setCachePolicy(CachePolicy domainCache) {
    this.domainCache = domainCache;
    this.log.debug("CachePolicy set to: " + domainCache);
  }
  
  public void flushCache() {
    if (this.domainCache != null)
      this.domainCache.flush(); 
  }
  
  public String getSecurityDomain() {
    return this.securityDomain;
  }
  
  public Subject getActiveSubject() {
    return SecurityAssociation.getSubject();
  }
  
  public boolean isValid(Principal principal, Object credential) {
    return isValid(principal, credential, null);
  }
  
  public boolean isValid(Principal principal, Object credential, Subject activeSubject) {
    DomainInfo cacheInfo = getCacheInfo(principal, true);
    if (this.trace)
      this.log.trace("Begin isValid, principal:" + principal + ", cache info: " + cacheInfo); 
    boolean isValid = false;
    if (cacheInfo != null) {
      isValid = validateCache(cacheInfo, credential, activeSubject);
      if (cacheInfo != null)
        cacheInfo.release(); 
    } 
    if (!isValid)
      isValid = authenticate(principal, credential, activeSubject); 
    if (this.trace)
      this.log.trace("End isValid, " + isValid); 
    return isValid;
  }
  
  public Principal getPrincipal(Principal principal) {
    Principal result = principal;
    synchronized (this.domainCache) {
      DomainInfo info = getCacheInfo(principal, false);
      if (this.trace)
        this.log.trace("getPrincipal, cache info: " + info); 
      if (info != null) {
        result = DomainInfo.access$000(info);
        if (result == null)
          result = principal; 
        info.release();
      } 
    } 
    return result;
  }
  
  public boolean doesUserHaveRole(Principal principal, Set rolePrincipals) {
    boolean hasRole = false;
    Subject subject = SubjectActions.getActiveSubject();
    if (subject != null) {
      if (this.trace)
        this.log.trace("doesUserHaveRole(Set), subject: " + subject); 
      Group roles = getSubjectRoles(subject);
      if (this.trace)
        this.log.trace("roles=" + roles); 
      if (roles != null) {
        Iterator iter = rolePrincipals.iterator();
        while (!hasRole && iter.hasNext()) {
          Principal role = iter.next();
          hasRole = doesRoleGroupHaveRole(role, roles);
          if (this.trace)
            this.log.trace("hasRole(" + role + ")=" + hasRole); 
        } 
      } 
      if (this.trace)
        this.log.trace("hasRole=" + hasRole); 
    } 
    return hasRole;
  }
  
  public boolean doesUserHaveRole(Principal principal, Principal role) {
    boolean hasRole = false;
    Subject subject = SubjectActions.getActiveSubject();
    if (subject != null) {
      if (this.trace)
        this.log.trace("doesUserHaveRole(Principal), subject: " + subject); 
      Group roles = getSubjectRoles(subject);
      if (roles != null)
        hasRole = doesRoleGroupHaveRole(role, roles); 
    } 
    return hasRole;
  }
  
  public Set getUserRoles(Principal principal) {
    HashSet userRoles = null;
    Subject subject = SubjectActions.getActiveSubject();
    if (subject != null) {
      if (this.trace)
        this.log.trace("getUserRoles, subject: " + subject); 
      Group roles = getSubjectRoles(subject);
      if (roles != null) {
        userRoles = new HashSet();
        Enumeration members = roles.members();
        while (members.hasMoreElements()) {
          Principal role = members.nextElement();
          userRoles.add(role);
        } 
      } 
    } 
    return userRoles;
  }
  
  protected boolean doesRoleGroupHaveRole(Principal role, Group userRoles) {
    if (role instanceof org.jboss.security.NobodyPrincipal)
      return false; 
    boolean isMember = userRoles.isMember(role);
    if (!isMember)
      isMember = role instanceof org.jboss.security.AnybodyPrincipal; 
    return isMember;
  }
  
  private boolean authenticate(Principal principal, Object credential, Subject theSubject) {
    Subject subject = null;
    boolean authenticated = false;
    LoginException authException = null;
    try {
      LoginContext lc = defaultLogin(principal, credential);
      subject = lc.getSubject();
      if (subject != null) {
        if (theSubject != null) {
          SubjectActions.copySubject(subject, theSubject);
        } else {
          theSubject = subject;
        } 
        authenticated = true;
        updateCache(lc, subject, principal, credential);
      } 
    } catch (LoginException e) {
      if ((principal != null && principal.getName() != null) || this.trace)
        this.log.trace("Login failure", e); 
      authException = e;
    } 
    SubjectActions.setContextInfo("org.jboss.security.exception", authException);
    return authenticated;
  }
  
  private LoginContext defaultLogin(Principal principal, Object credential) throws LoginException {
    Object[] securityInfo = { principal, credential };
    CallbackHandler theHandler = null;
    try {
      theHandler = (CallbackHandler)this.handler.getClass().newInstance();
      this.setSecurityInfo.invoke(theHandler, securityInfo);
    } catch (Throwable e) {
      if (this.trace)
        this.log.trace("Failed to create/setSecurityInfo on handler", e); 
      LoginException le = new LoginException("Failed to setSecurityInfo on handler");
      le.initCause(e);
      throw le;
    } 
    Subject subject = new Subject();
    LoginContext lc = null;
    if (this.trace)
      this.log.trace("defaultLogin, principal=" + principal); 
    lc = SubjectActions.createLoginContext(this.securityDomain, subject, theHandler);
    lc.login();
    if (this.trace)
      this.log.trace("defaultLogin, lc=" + lc + ", subject=" + SubjectActions.toString(subject)); 
    return lc;
  }
  
  private boolean validateCache(DomainInfo info, Object credential, Subject theSubject) {
    if (this.trace) {
      StringBuffer tmp = new StringBuffer("Begin validateCache, info=");
      tmp.append(info.toString());
      tmp.append(";credential.class=");
      if (credential != null) {
        Class c = credential.getClass();
        tmp.append(c.getName());
        tmp.append('@');
        tmp.append(System.identityHashCode(c));
      } else {
        tmp.append("null");
      } 
      this.log.trace(tmp.toString());
    } 
    Object subjectCredential = DomainInfo.access$100(info);
    boolean isValid = false;
    if (credential == null || subjectCredential == null) {
      isValid = (credential == null && subjectCredential == null);
    } else if (subjectCredential.getClass().isAssignableFrom(credential.getClass())) {
      if (subjectCredential instanceof Comparable) {
        Comparable c = (Comparable)subjectCredential;
        isValid = (c.compareTo(credential) == 0);
      } else if (subjectCredential instanceof char[]) {
        char[] a1 = (char[])subjectCredential;
        char[] a2 = (char[])credential;
        isValid = Arrays.equals(a1, a2);
      } else if (subjectCredential instanceof byte[]) {
        byte[] a1 = (byte[])subjectCredential;
        byte[] a2 = (byte[])credential;
        isValid = Arrays.equals(a1, a2);
      } else if (subjectCredential.getClass().isArray()) {
        Object[] a1 = (Object[])subjectCredential;
        Object[] a2 = (Object[])credential;
        isValid = Arrays.equals(a1, a2);
      } else {
        isValid = subjectCredential.equals(credential);
      } 
    } 
    if (isValid)
      if (theSubject != null)
        SubjectActions.copySubject(DomainInfo.access$200(info), theSubject);  
    if (this.trace)
      this.log.trace("End validateCache, isValid=" + isValid); 
    return isValid;
  }
  
  private DomainInfo getCacheInfo(Principal principal, boolean allowRefresh) {
    if (this.domainCache == null)
      return null; 
    DomainInfo cacheInfo = null;
    synchronized (this.domainCache) {
      if (allowRefresh == true) {
        cacheInfo = (DomainInfo)this.domainCache.get(principal);
      } else {
        cacheInfo = (DomainInfo)this.domainCache.peek(principal);
      } 
      if (cacheInfo != null)
        cacheInfo.acquire(); 
    } 
    return cacheInfo;
  }
  
  private Subject updateCache(LoginContext lc, Subject subject, Principal principal, Object credential) {
    if (this.domainCache == null)
      return subject; 
    long lifetime = 0L;
    if (this.domainCache instanceof TimedCachePolicy) {
      TimedCachePolicy cache = (TimedCachePolicy)this.domainCache;
      lifetime = cache.getDefaultLifetime();
    } 
    DomainInfo info = new DomainInfo(lifetime);
    DomainInfo.access$302(info, lc);
    DomainInfo.access$202(info, new Subject());
    SubjectActions.copySubject(subject, DomainInfo.access$200(info), true);
    DomainInfo.access$102(info, credential);
    if (this.trace)
      this.log.trace("updateCache, inputSubject=" + SubjectActions.toString(subject) + ", cacheSubject=" + SubjectActions.toString(DomainInfo.access$200(info))); 
    Set subjectGroups = subject.getPrincipals(Group.class);
    Iterator iter = subjectGroups.iterator();
    while (iter.hasNext()) {
      Group grp = iter.next();
      String name = grp.getName();
      if (name.equals("CallerPrincipal")) {
        Enumeration members = grp.members();
        if (members.hasMoreElements())
          DomainInfo.access$002(info, members.nextElement()); 
      } 
    } 
    if (principal == null && DomainInfo.access$000(info) == null) {
      Set subjectPrincipals = subject.getPrincipals(Principal.class);
      iter = (Iterator)subjectPrincipals.iterator();
      while (iter.hasNext()) {
        Principal p = (Principal)iter.next();
        if (!(p instanceof Group))
          DomainInfo.access$002(info, p); 
      } 
    } 
    synchronized (this.domainCache) {
      if (this.domainCache.peek(principal) != null)
        this.domainCache.remove(principal); 
      this.domainCache.insert(principal, info);
      if (this.trace)
        this.log.trace("Inserted cache info: " + info); 
    } 
    return DomainInfo.access$200(info);
  }
  
  private Group getSubjectRoles(Subject theSubject) {
    Set subjectGroups = theSubject.getPrincipals(Group.class);
    Iterator iter = subjectGroups.iterator();
    Group roles = null;
    while (iter.hasNext()) {
      Group grp = iter.next();
      String name = grp.getName();
      if (name.equals("Roles"))
        roles = grp; 
    } 
    return roles;
  }
}
